/*
 * Copyright (c) 2006 Markus Fisch <mf@markusfisch.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of Markus Fisch nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef erc_Compiler_h
#define erc_Compiler_h

#include <string>
#include <sstream>

namespace erc
{
	/**
	 * Simple resource compiler
	 *
	 * @author mf@markusfisch.de
	 * @version 0.5.0
	 */
	class Compiler
	{
		public:
			enum Format
			{
				STL,
				wxWidgets
			};

			Compiler() :
				offset( 0 ),
				fileName( "Resource" ),
				format( STL ) {}
			virtual ~Compiler();
			inline void setFileName( const char *f ) { fileName = f; flush(); }
			inline const std::string &getFileName() const { return fileName; }
			inline void setFormat( Format f ) { format = f; }
			inline const Format &getFormat() const { return format; }
			virtual void add( const char * );

		protected:
			std::ostringstream data;
			std::ostringstream index;
			int offset;

			virtual int hash( const char * ) const;
			virtual void flush();

		private:
			enum
			{
				HashMax = 600011
			};

			std::string fileName;
			Format format;
	};
}

#endif
