/*
 * Copyright (c) 2006 Markus Fisch <mf@markusfisch.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of Markus Fisch nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
#include "Compiler.h"

#include <iostream>
#include <fstream>
#include <string>

using namespace std;

int main( int argc, char **argv )
{
	erc::Compiler c;

	try
	{
		for( const char *binary = *argv; --argc; )
			if( **(++argv) != '-' )
				c.add( *argv );
			else
				switch( *(*argv+1) )
				{
					default:
						cerr << "unknown option '"
							<< *(*argv+1)
							<< "'"
							<< endl;
						break;
					case 'h':
					case '?':
						cout << "usage: " << binary << " [hviof] FILE..."
							<< endl
							<< "  -h         "
							<< "print this help and exit"
							<< endl
							<< "  -v         "
							<< "print version and exit"
							<< endl
							<< "  -i FILE    "
							<< "read input files from this file"
							<< endl
							<< "  -o FILE    "
							<< "use this filename for output (.cpp/.h)"
								<< ", default is \"Resources\""
							<< endl
							<< "  -f FORMAT  "
							<< "set output format: STL (default) or wxWidgets"
							<< endl;
						return 0;
					case 'v':
						cout << "Embedded Resources Compiler 0.4.0 "
							<< "(c) 2007 Markus Fisch <mf@markusfisch.de>"
							<< endl;
						return 0;
					case 'i':
						if( !--argc )
							throw "missing argument for -i option !";
						else
						{
							ifstream in( *++argv );

							if( !in ||
								!in.good() )
								throw "can't open input file !";

							for( string s; getline( in, s ); )
							{
								// don't use lines that start by #
								string::size_type p = 
									s.find_first_not_of( " \t" );

								if( p == string::npos ||
									s[p] == '#' )
									continue;

								c.add( s.c_str() );
							}

							in.close();
						}
						break;
					case 'o':
						if( !--argc )
							throw "missing argument for -o option !";
						c.setFileName( *++argv );
						break;
					case 'f':
						if( !--argc )
							throw "missing argument for -f option !";
						else
						{
							++argv;

							if( !strcmp( *argv, "wxWidgets" ) )
								c.setFormat( erc::Compiler::wxWidgets );
							else if( !strcmp( *argv, "STL" ) )
								c.setFormat( erc::Compiler::STL );
							else
								throw "unknown output format !";
						}
						break;
				}
	}
	catch( const char *e )
	{
		cerr << "error: " << e << endl;
	}

	return 0;
}
